#include <sta/stm32/gpio_pin.hpp>
#ifdef STA_STM32_GPIO_ENABLE

#include <sta/assert.hpp>
#include <sta/lang.hpp>


namespace sta
{
	STM32GpioPin::STM32GpioPin(GPIO_TypeDef * port, uint16_t pin)
		: port_{port}, pin_{pin}
	{
		STA_ASSERT(port != nullptr);
	}

	void STM32GpioPin::setState(GpioPinState state)
	{
		HAL_GPIO_WritePin(port_, pin_, (state == GpioPinState::LOW) ? GPIO_PIN_RESET : GPIO_PIN_SET);
	}

	GPIO_TypeDef * STM32GpioPin::getPort() const
	{
		return port_;
	}

	uint16_t STM32GpioPin::getPin() const
	{
		return pin_;
	}

	uint8_t STM32GpioPin::getPortIndex() const
	{
		return GPIO_GET_INDEX(port_);
	}


	bool isInterruptEdge(const STM32GpioPin & gpioPin, InterruptEdge edge)
	{
		uint32_t pin = gpioPin.getPin();

		for (uint32_t i = 0; i < 8 * sizeof(pin); ++i)
		{
			uint32_t ioPos = 1U << i;
			if (pin & ioPos)
			{
				// Check input mode
				uint32_t mode = (gpioPin.getPort()->MODER >> (2U * i)) & GPIO_MODE;
				if (mode != MODE_INPUT)
				{
					return false;
				}

				// Is EXTI configured?
				if (EXTI->IMR & ioPos)
				{
					bool rising  = (EXTI->RTSR & ioPos);
					bool falling = (EXTI->FTSR & ioPos);

					switch (edge)
					{
					case InterruptEdge::RISING:
						return rising;

					case InterruptEdge::FALLING:
						return falling;

					case InterruptEdge::BOTH:
						return rising && falling;

					default:
						STA_ASSERT(false);
						STA_UNREACHABLE();
					}
				}
			}
		}

		return false;
	}
} // namespace sta


#endif // STA_STM32_GPIO_ENABLE
